// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:io';

import 'package:wyatt_continuous_deployment/src/core/constants/app_constants.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/core/task/task_cubit.dart';
import 'package:wyatt_continuous_deployment/src/core/utils/logging.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/config/appstore_connect_config.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/context.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/decrypt_keys_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/fastlane.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/fastlane/fastlane_deploy_pilot_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/config/get_ios_config_usecase.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/decrypt_keys/decrypt_keys_usecase.dart';

part 'deploy_ios_state.dart';

class DeployIosTask extends TaskCubit {
  factory DeployIosTask() => DeployIosTask._(
        getIosConfigUsecase: getIt(),
        decryptKeysUsecase: getIt(),
        fastlane: getIt(),
        checkToolsUsecase: getIt(),
        checkConfigFileUsecase: getIt(),
      );

  DeployIosTask._({
    required GetIosConfigUsecase getIosConfigUsecase,
    required DecryptKeysUsecase decryptKeysUsecase,
    required Fastlane fastlane,
    required super.checkToolsUsecase,
    required super.checkConfigFileUsecase,
  })  : _getIosConfigUsecase = getIosConfigUsecase,
        _decryptKeysUsecase = decryptKeysUsecase,
        _fastlane = fastlane;

  final GetIosConfigUsecase _getIosConfigUsecase;
  final DecryptKeysUsecase _decryptKeysUsecase;
  final Fastlane _fastlane;

  @override
  Future<void> onRun(Context ctx) async {
    // 1. Get iOS config
    emit(DeployIosStateGetIosConfig());
    final getIosConfigResult = await _getIosConfigUsecase(ctx.configFile.path);
    if (getIosConfigResult.isErr) {
      return emit(
        TaskStateError(message: getIosConfigResult.err.toString()),
      );
    }

    // 2. Decrypt iOS keys
    emit(DeployIosStatedecryptIosKeys());
    final decryptIosKeyResult = await _decryptKeysUsecase(
      DecryptKeysParameters(
        encryptedFilePath: getIosConfigResult.ok!.encryptedKeyFile.path,
        passphrase:
            ctx.environment[AppConstants.iosKeysSecretPassphrase].toString(),
      ),
    );
    if (decryptIosKeyResult.isErr) {
      return emit(TaskStateError(message: decryptIosKeyResult.err.toString()));
    }

    // 5. Deploy to AppStore Connect
    emit(DeployIosStateDeployToAppStoreConnect());
    final deployResult = await _fastlane.deployAppStore(
      FastlaneDeployPilotParameters(
        appStoreConnectConfig: AppStoreConnectConfig.fromConfig(
          getIosConfigResult.ok!,
        ),
        developerAppId: getIosConfigResult.ok!.developerAppId,
        developerAppIdentifier: getIosConfigResult.ok!.developerAppIdentifier,
        ipaPath: _findFirstIpaFilePath('build/ios/ipa') ?? '',
      ),
    );
    if (deployResult.isErr) {
      return emit(TaskStateError(message: deployResult.err.toString()));
    }
  }

  String? _findFirstIpaFilePath(String folderPath) {
    final Directory directory = Directory(folderPath);

    if (!directory.existsSync()) {
      logger.detail('No IPA file found in $folderPath');
      return null;
    }

    final List<FileSystemEntity> files = directory.listSync();

    for (final FileSystemEntity file in files) {
      if (file is File && file.path.endsWith('.ipa')) {
        logger.detail('Found IPA file: ${file.path}');
        return file.path;
      }
    }

    logger.detail('No IPA file found in $folderPath');
    return null;
  }
}
